# -*- coding: utf-8 -*-
"""
TP sur la recherche textuelle
"""

# 2

def identique_0(strg: str, substrg:str, i:int, k:int)->bool:
    return strg[i:i+k]==substrg


def identique(strg: str, substrg:str, i:int, k:int)->bool:
    """
    Teste la présence du motif dans strg entre les positions i et i + k - 1

    Parameters
    ----------
    strg : str
        chaine de caractères dans laquelle on cherche le motif
    substrg : str
        motif recherché
    i : int
        indice dans strg à partir duquel on teste la présence du motif
    k : int
        longueur du motif substrg


    Returns
    -------
    bool
        True si strg[i:i+k] == substrg, False sinon
   
    """

    

# 3

def recherche_naive(strg : str, substrg : str)->int: 
    """
    Recherche la sous-chaine substrg dans la chaine strg

    Parameters
    ----------
    strg : str
        chaine de caractères dans laquelle on cherche le motif
    substrg : str
        motif recherché

    Returns
    -------
    int
        indice du première caractère de la chaine strg où le
        motif est trouvé
        -1 en cas d'échec de la recherche
    
    Examples
    --------
    >>> recherche_naive("anticonstitutionnellement", "elle")
    17
    
    >>> recherche_naive("abcdabefabg", "ab")
    0
    
    >>> recherche_naive("abcdefg", "g")
    6
    
    >>> recherche_naive('deaaadcebbdbdcc', 'bdcc')
    11
    
    >>> recherche_naive("abcdefg", "cdf")
    -1
    
    >>> recherche_naive("abcdefg", "gdf")
    -1

    """


# 4

def recherche_naive_2(strg : str, substrg : str)->int: 
    "Même préconditions et postconditions que recherche_naive"

    


# 4 autre solution (plus rapide mais moins pédagogique)

def recherche_naive_2_bis(strg : str, substrg : str)->int: 
    "Même préconditions et postconditions que recherche_naive"
     


# 5

def identique_1(strg: str, substrg:str, i:int, k:int)->bool:
    "Même préconditions et postconditions que identique"

    


def horspool_1(strg: str, substrg: str)->int: 
    "Même préconditions et postconditions que recherche_naive"
     


# 6

def identique_2(strg: str, substrg:str, i:int, k:int)->(bool, int):
    """
    Examples
    -------
    >>> identique_2("abcdefgh", "abc", 0, 3)
    (True, 0)
    
    >>> identique_2("abcdefgh", "abc", 1, 3)
    (False, 3)
    
    >>> identique_2("abcaefgh", "abc", 1, 3)
    (False, 1)
    
    >>> identique_2("abcaeabc", "abc", 5, 3)
    (True, 0)
    
    """



def horspool_2(strg: str, substrg: str)->int: 
    """
    Même préconditions et postconditions que recherche_naive
    """
    


# 7

def premiere_table(substrg: str)->dict:
    """
    Renvoie la première table de sauts

    Parameters
    ----------
    substrg : str
        chaine de caractères recherchée (motif)

    Returns
    -------
    dict
        première table de sauts associant à chaque lettre du motif le 
        nombre de sauts
    
    Examples
    --------
    >>> premiere_table("aloha")
    {'h': 1, 'o': 2, 'l': 3, 'a': 4}
    
    
    >>> premiere_table("TARTEMPION")
    {'O': 1, 'I': 2, 'P': 3, 'M': 4, 'E': 5, 'T': 6, 'R': 7, 'A': 8}
    

    """



# 8

def identique_3(strg: str, substrg:str, i:int, k:int, table:dict)->(bool, int):
    """
    Examples
    -------
    >>> identique_3("abcdefgh", "abc", 0, 3, premiere_table("abc"))
    (True, 0)
    
    >>> identique_3("yamomoto", "moto", 1, 4, premiere_table("moto"))
    (False, 3)
    
    >>> identique_3("bonjour je m'appelle emilie jolie", "jolie", 22, 5, premiere_table("jolie"))
    (False, 1)
    
    >>> identique_3('cabdbadcaddccdabeee', 'abeeda', 9, 6, premiere_table('abeeda'))
    (False, 4)
    
    >>> identique_3("abcaeabc", "abc", 5, 3, premiere_table("abc"))
    (True, 0)
    
    >>> identique_3("NPANMANAM", "NNAAMAN", 0, 7, premiere_table("NNAAMAN"))
    (False, 2)
    
    """



def horspool_3(strg: str, substrg: str)->int: 
    """
    Algorithme de Boyer-Moore-Horspool
    Version simplifiée à une seule table de l'algorithme de Boyer-Moore
    Même préconditions et postconditions que recherche_naive"
    
    """



# 9

def recherche_suffixe_0(substrg: str, m: int)->bool:
    """
    Recherche la présence d'une précédente occurrence du suffixe identifié par m
    précédé d'un autre caractère que celui qui précède le suffixe
    
    Parameters
    ----------
    substrg : str
        motif recherché
    m : int
        entier compris entre 0 et len(substrg) - 1 permettant d'identifier le suffixe
        si m = 1, le suffixe est la dernière lettre du motif substrg
        si m = 2, le suffixe est composé des deux dernières lettres du motif...
    
    Returns
    -------
    bool
        indique la présence ou pas d'une autre occurrence
    
    Examples
    --------
    >>> recherche_suffixe_0("dabcda", 1)
    False
    
    >>> recherche_suffixe_0("cabcda", 1)
    True
    
    >>> recherche_suffixe_0("abababab", 3)
    False
    
    >>> recherche_suffixe_0("babababab", 3)
    True
    
    >>> recherche_suffixe_0("oblobababab", 3)
    True
        
    """




def recherche_suffixe_1(substrg: str, m: int)->int:
    """
    Calcule le décalage à appliquer pour la seconde table de sauts
    
    Parameters
    ----------
    substrg : str
        motif recherché
    m : int
        entier compris entre 0 et len(substrg) - 1 permettant d'identifier le suffixe
        si m = 1, le suffixe est la dernière lettre du motif substrg
        si m = 2, le suffixe est composé des deux dernières lettres du motif...
    
    Returns
    -------
    int
        décalage à appliquer
    
    Examples
    --------
    >>> recherche_suffixe_1("conda", 0)
    1
    
    >>> recherche_suffixe_1("conda", 1)
    5
    
    >>> recherche_suffixe_1("ondaconda", 2)
    9
    
    >>> recherche_suffixe_1("anaunasauna", 2)
    8
    
    >>> recherche_suffixe_1("papopapipapipapi", 4)
    8
    
    >>> recherche_suffixe_1("babababab", 3)
    6
    
    >>> recherche_suffixe_1("oblobababab", 3)
    4
    
    
    """

      

def recherche_suffixe_2(substrg: str, m: int)->int:
    """
    Calcule le décalage à appliquer pour la seconde table de sauts
    
    Parameters
    ----------
    substrg : str
        motif recherché
    m : int
        entier compris entre 0 et len(substrg) - 1 permettant d'identifier le suffixe
        si m = 1, le suffixe est la dernière lettre du motif substrg
        si m = 2, le suffixe est composé des deux dernières lettres du motif...
    
    Returns
    -------
    int
        décalage à appliquer
    
    Examples
    --------
    >>> recherche_suffixe_2("conda", 0)
    1
    
    >>> recherche_suffixe_2("conda", 1)
    5
    
    >>> recherche_suffixe_2("ondaconda", 2)
    9
    
    >>> recherche_suffixe_2("anaunasauna", 2)
    8
    
    >>> recherche_suffixe_2("papopapipapipapi", 4)
    8
    
    >>> recherche_suffixe_2("pipapipapipapi", 4)
    12
    
    >>> recherche_suffixe_2("BAMBAM", 4)
    3
    
    >>> recherche_suffixe_2("ANPANMAN", 7)
    6
    
    """




# 10
          
MAX_SUFFIX_LENGTH = 20

        
def seconde_table(substrg: str)->list:
    """
    Renvoie la seconde table de sauts 
    
    Parameters
    ----------
    substrg : str
        chaine de caractères recherchée (motif)

    Returns
    -------
    list
        seconde table de sauts
    
    Example
    --------
    >>> seconde_table("ANPANMAN")
    [1, 8, 3, 6, 6, 6, 6, 6]
  
    >>> seconde_table("01010")
    [1, 4, 4, 2, 2]
    
    >>> seconde_table("GCAGAGAG")
    [1, 7, 4, 7, 2, 7, 7, 7]

    >>> seconde_table("GCAGAGAGAGATGAAGATAGAAGATTAAGGCCA")
    [1, 5, 30, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33]

    """

        

def identique_4(strg: str, substrg:str, i:int, k:int, table_1:dict, table_2: list)->(bool, int):
    """
    Examples
    -------
    >> identique_4("yamomoto", "moto", 1, 4, premiere_table("moto"), seconde_table("moto"))
    (False, 3)
    
    >>> identique_4("bonjour je m'appelle emilie jolie", "jolie", 22, 5, premiere_table("jolie"), seconde_table("jolie"))
    (False, 5)
    
    >>> identique_4('cabdbadcaddccdabeee', 'abeeda', 9, 6, premiere_table('abeeda'), seconde_table('abeeda'))
    (False, 5)
    
    >>> identique_4("abcaeabc", "abc", 5, 3, premiere_table("abc"), seconde_table("abc"))
    (True, 0)
    
    >>> identique_4("NPANMANAM", "NNAAMAN", 0, 7, premiere_table("NNAAMAN"), seconde_table("NNAAMAN"))
    (False, 6)
    
    """




def boyer_moore(strg: str, substrg: str)->int: 
    """
    Algorithme de Boyer-Moore
    Même préconditions et postconditions que recherche_naive"
    
    """
   







